/* 
 * AndSys 0.1.l.11 Copyright (C) 2012, 2013 Giulio Orru'
 * 
 * Licensed under the MIT license:
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package net.sourceforge.andsys;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.text.Html;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.Comparator;

public class ActivityPackages extends android.app.ListActivity implements
		OnItemClickListener, OnItemSelectedListener {

	private static final int MENU_ABOUT = 1;
	private static final int MENU_EXIT = 2;

	/**
	 * Called when the activity is first created.This is where you should do all
	 * of your normal static set up: create views, bind data to lists, etc. This
	 * method also provides you with a Bundle containing the activity's
	 * previously frozen state, if there was one. Always followed by onStart().
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_packages);

	}

	/**
	 * Called after your activity has been stopped, prior to it being started
	 * again. Always followed by onStart()
	 */
	protected void onRestart() {
		super.onRestart();
	}

	/**
	 * Called when the activity is becoming visible to the user. Followed by
	 * onResume() if the activity comes to the foreground, or onStop() if it
	 * becomes hidden.
	 */
	protected void onStart() {
		super.onStart();

		Application application = (Application) this.getApplication();

		// THERE ARE 2 COPIES OF THIS CODE: BEGIN
		{
			// * Order by.. (in the global scope)
			Comparator<Package> pc;
			if (application.orderby == Application.ORDERBY_PACKAGE) {
				pc = new Package.ComparatorPackage();
			} else if (application.orderby == Application.ORDERBY_PACKAGE_DESC) {
				pc = new Package.ComparatorPackage();
				pc = Collections.reverseOrder(pc);
			} else if (application.orderby == Application.ORDERBY_NAME) {
				pc = new Package.ComparatorName();
			} else if (application.orderby == Application.ORDERBY_NAME_DESC) {
				pc = new Package.ComparatorName();
				pc = Collections.reverseOrder(pc);
			} else if (application.orderby == Application.ORDERBY_INSTALLED) {
				pc = new Package.ComparatorInstalled();
			} else {
				pc = new Package.ComparatorInstalled();
				pc = Collections.reverseOrder(pc);
			}
			Collections.sort(application.apList, pc);

			// * View packages list (from the global scope)
			ListAdapter listAdapter;
			listAdapter = new PackageAdapter(this.getApplicationContext(),
					R.layout.package_adapter_list_item, application.apList);
			// The AdapterView does nothing more than manage visibility.
			setListAdapter(listAdapter);
		}
		// END

		ListView listView = (ListView) findViewById(android.R.id.list);
		listView.setOnItemClickListener(this);
	}

	/**
	 * Called when the activity will start interacting with the user. At this
	 * point your activity is at the top of the activity stack, with user input
	 * going to it. Always followed by onPause().
	 */
	protected void onResume() {
		super.onResume();

		Application asApplication = (Application) this.getApplication();

		// Spinner
		{
			Spinner spinner1 = (Spinner) findViewById(R.id.spinner1);

			// Create an ArrayAdapter using the string array and a default
			// spinner layout
			ArrayAdapter<CharSequence> adapter = ArrayAdapter
					.createFromResource(this, R.array.package_comparator,
							android.R.layout.simple_spinner_item);

			// Specify the layout to use when the list of choices appears
			adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

			// Apply the adapter to the spinner
			spinner1.setAdapter(adapter);
			spinner1.setOnItemSelectedListener(this);
			spinner1.setSelection(asApplication.orderby);
		}
	}

	/**
	 * Called when the system is about to start resuming a previous activity.
	 * This is typically used to commit unsaved changes to persistent data, stop
	 * animations and other things that may be consuming CPU, etc.
	 * Implementations of this method must be very quick because the next
	 * activity will not be resumed until this method returns. Followed by
	 * either onResume() if the activity returns back to the front, or onStop()
	 * if it becomes invisible to the user.
	 */
	protected void onPause() {
		super.onPause();
	}

	/**
	 * Called when the activity is no longer visible to the user, because
	 * another activity has been resumed and is covering this one. This may
	 * happen either because a new activity is being started, an existing one is
	 * being brought in front of this one, or this one is being destroyed.
	 * Followed by either onRestart() if this activity is coming back to
	 * interact with the user, or onDestroy() if this activity is going away.
	 */
	protected void onStop() {
		super.onStop();
	}

	/**
	 * The final call you receive before your activity is destroyed. This can
	 * happen either because the activity is finishing (someone called finish()
	 * on it, or because the system is temporarily destroying this instance of
	 * the activity to save space. You can distinguish between these two
	 * scenarios with the isFinishing() method.
	 */
	protected void onDestroy() {
		super.onDestroy();
	}

	/**
	 * Initialize the contents of the Activity's standard options menu. The
	 * options menu is shown when you press the "Menu" button on the device.
	 * 
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuItem miAbout;
		MenuItem miExit;
		miAbout = menu.add(0, MENU_ABOUT, 0, R.string.menu_about);
		miAbout.setIcon(android.R.drawable.ic_menu_info_details);
		miExit = menu.add(0, MENU_EXIT, 0, R.string.menu_exit);
		miExit.setIcon(android.R.drawable.ic_menu_close_clear_cancel);
		return true;
	}

	/**
	 * This hook is called whenever an item in your options menu is selected.
	 * 
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case MENU_ABOUT:
			AlertDialog.Builder adbAbout = new AlertDialog.Builder(this);
			String text = getString(R.string.readme_styledtext);
			CharSequence styledText = Html.fromHtml(text);
			adbAbout.setMessage(styledText);
			//adbAbout.setMessage(getFileContentString(getApplicationContext(),
			//		R.raw.txt_readme));
			adbAbout.setCancelable(true);
			// web link to donation page
			adbAbout.setPositiveButton(R.string.button_donate,
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							// donation uri: Andsys
							Uri uri = Uri.parse(getFileContentString(
									getApplicationContext(),
									R.raw.uri_donate075));
							Intent intent = new Intent();
							intent.setAction(Intent.ACTION_VIEW);
							intent.addCategory(Intent.CATEGORY_BROWSABLE);
							intent.setData(uri);
							startActivity(intent);
							dialog.dismiss();
						}
					});
			// web link to license
			adbAbout.setNeutralButton(R.string.button_license,
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							// donation uri: Andsys
							Uri uri = Uri.parse(getFileContentString(
									getApplicationContext(),
									R.raw.uri_license_mit));
							Intent intent = new Intent();
							intent.setAction(Intent.ACTION_VIEW);
							intent.addCategory(Intent.CATEGORY_BROWSABLE);
							intent.setData(uri);
							startActivity(intent);
							dialog.dismiss();
						}
					});
			// hide dialog
			adbAbout.setNegativeButton(android.R.string.cancel,
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							dialog.dismiss();
						}
					});
			// create dialog
			AlertDialog adAbout = adbAbout.create();
			adAbout.setTitle(R.string.app_name);
			adAbout.setIcon(R.drawable.ic_launcher);
			adAbout.show();
			return true;
		case MENU_EXIT:
			AlertDialog.Builder adbExit = new AlertDialog.Builder(this);
			adbExit.setMessage(R.string.dialog_exit);
			adbExit.setCancelable(true);
			// finish the application
			adbExit.setPositiveButton(android.R.string.ok,
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							finish();
						}
					});
			// hide dialog
			adbExit.setNegativeButton(android.R.string.no,
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int id) {
							dialog.dismiss();
						}
					});
			// create dialog
			AlertDialog adExit = adbExit.create();
			adExit.setTitle(R.string.menu_exit);
			adExit.setIcon(android.R.drawable.ic_menu_close_clear_cancel);
			adExit.show();
			return true;
		}
		return false;
	}

	/**
	 * Returns the resource file in a String
	 * 
	 * @param context
	 *            Current state of the application/object. Typically you call it
	 *            to get information regarding another part of your program. You
	 *            can get the context by invoking getApplicationContext(),
	 *            getContext(), getBaseContext() or this (when in the activity
	 *            class).
	 * @param resource
	 *            It is the id of the resource.
	 * @return Get a text file contents as a String.
	 */
	private static String getFileContentString(final Context context,
			final int resource) {
		StringBuilder fileRow = new StringBuilder();

		try {
			String row;
			InputStream is = context.getResources().openRawResource(resource);
			BufferedReader br = new BufferedReader(new InputStreamReader(is));

			while ((row = br.readLine()) != null) {
				fileRow.append(row + '\n');
			}
		} catch (IOException e) {
			return null;
		}

		return fileRow.toString();
	}

	@SuppressLint("InlinedApi")
	public void onItemClick(AdapterView<?> parent, View view, int position,
			long id) {
		
		Application application = (Application) this.getApplication();
		String packageFullname = application.apList.get(position).getPackage();
		Toast toast = Toast.makeText(parent.getContext(), packageFullname,
				Toast.LENGTH_SHORT);
		toast.show();

		Intent uninstallIntent;
        Uri uninstalPackageUri = Uri.parse("package:" + packageFullname);
        if (android.os.Build.VERSION.SDK_INT >= 9) 
        {
    		uninstallIntent = new Intent(android.provider.Settings.ACTION_APPLICATION_DETAILS_SETTINGS, uninstalPackageUri);
        } 
        else 
        {
            uninstallIntent = new Intent(Intent.ACTION_DELETE, uninstalPackageUri);
        }
        uninstallIntent.putExtra("package", packageFullname);
        startActivityForResult(uninstallIntent, 1);
	}


	/*
	 * OnItemSelectedListener: OnItemSelectedListener
	 */
	public void onItemSelected(AdapterView<?> parent, View view, int position,
			long id) {
		Toast toast = Toast.makeText(parent.getContext(), parent
				.getItemAtPosition(position).toString(), Toast.LENGTH_SHORT);
		toast.show();
		// Set Order by.. (in the global scope)
		if (position >= 0 && position < Application.COUNT_ORDERBY) {
			Application application = (Application) this.getApplication();
			if (position != application.orderby) {
				application.orderby = position;

				// THERE ARE 2 COPIES OF THIS CODE: BEGIN
				{
					// * Order by.. (in the global scope)
					Comparator<Package> pc;
					if (application.orderby == Application.ORDERBY_PACKAGE) {
						pc = new Package.ComparatorPackage();
					} else if (application.orderby == Application.ORDERBY_PACKAGE_DESC) {
						pc = new Package.ComparatorPackage();
						pc = Collections.reverseOrder(pc);
					} else if (application.orderby == Application.ORDERBY_NAME) {
						pc = new Package.ComparatorName();
					} else if (application.orderby == Application.ORDERBY_NAME_DESC) {
						pc = new Package.ComparatorName();
						pc = Collections.reverseOrder(pc);
					} else if (application.orderby == Application.ORDERBY_INSTALLED) {
						pc = new Package.ComparatorInstalled();
					} else {
						pc = new Package.ComparatorInstalled();
						pc = Collections.reverseOrder(pc);
					}
					Collections.sort(application.apList, pc);

					// * View packages list (from the global scope)
					ListAdapter listAdapter;
					listAdapter = new PackageAdapter(
							this.getApplicationContext(),
							R.layout.package_adapter_list_item,
							application.apList);
					// The AdapterView does nothing more than manage visibility.
					setListAdapter(listAdapter);
				}
				// END
			}
		}
	}

	/*
	 * OnItemSelectedListener: onNothingSelected
	 */
	@Override
	public void onNothingSelected(AdapterView<?> arg0) {
		// TODO Auto-generated method stub

	}

}
